/*
This is a part of the LiteStep Shell Source code.

Copyright (C) 1997-2001 The LiteStep Development Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/ 
/*
  Copyright (C) 1998-1999 Johan Redestig
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/****************************************************************************
****************************************************************************/
#include <algorithm>
#include "stdafx.h"
#include "PopupMaker.h"
#include "PopupMenu.h"
#include "DirectoryFolder.h"
#include "Popup.h"
PopupMaker* pPopupMaker;

void PopupBangCommand(HWND hCaller, const char *pszName, const char *pszArguments);

int initModuleEx(HWND ParentWnd, HINSTANCE dllInst, LPCSTR szPath)
{
	g_hResStrInstance = dllInst;
	CoInitialize(NULL);
	pPopupMaker = new PopupMaker();
	pPopupMaker->Initialize(ParentWnd, dllInst, szPath);

	AddBangCommandEx("!Popup", PopupBangCommand);
    AddBangCommand("!PopupDynamicFolder", DynamicBangCommand);
	return 0;
}

int quitModule(HINSTANCE dllInst)
{
	RemoveBangCommand("!PopupDynamicFolder");
    RemoveBangCommand("!Popup");
	CoUninitialize();
	delete pPopupMaker;

	return 1;
}

// Dummy startup routine that does almost nothing.
// This cuts out all of the standard startup code crud that
// bloats the DLL, and makes it take longer to load
BOOL __stdcall _DllMainCRTStartup(HINSTANCE hInst, DWORD fdwReason, LPVOID lpvRes)
{
	// We don't need thread notifications for what we're doing.  Thus, get
	// rid of them, thereby eliminating some of the overhead of this DLL
	DisableThreadLibraryCalls(hInst);

	return TRUE;
}

void ParseBangParameters(LPCSTR pszArguments, int& nX, int& nY, int& nAlignX, int& nAlignY)
{
	char szFirst[32];
	char szSecond[32];
	char szThird[32];
	char szFourth[32];

	char *rgszTokens[] = { szFirst, szSecond, szThird, szFourth };
	int nTokens = LCTokenize(pszArguments, rgszTokens, 4, 0);

	char *pszAlignX = 0;
	char *pszAlignY = 0;

	nAlignX = ALIGN_LEFT;
	nAlignY = ALIGN_TOP;

	POINT pt;
	GetCursorPos(&pt);

	nX = pt.x;
	nY = pt.y;

	if (nTokens >= 2)
	{
		if (isdigit(szFirst[0]) || szFirst[0] == '-' || strnicmp(szFirst, "X=", 2) == 0)
		{
			if (strnicmp(szFirst, "X=", 2) == 0)
				nX = atoi(szFirst + 2);
			else
				nX = atoi(szFirst);

			if (strnicmp(szSecond, "Y=", 2) == 0)
				nY = atoi(szSecond + 2);
			else
                nY = atoi(szSecond);
            
            nX = CONVERT_COORDINATE_X(nX);
            nY = CONVERT_COORDINATE_Y(nY);
            
            if (nTokens >= 4)
            {
                pszAlignX = szThird;
                pszAlignY = szFourth;
            }
        }
        else
        {
            pszAlignX = szFirst;
            pszAlignY = szSecond;
        }
        
        if (pszAlignX != 0 && pszAlignY != 0)
        {
            if (stricmp(pszAlignX, "center") == 0)
                nAlignX = ALIGN_CENTER;
            else if (stricmp(pszAlignX, "right") == 0)
                nAlignX = ALIGN_RIGHT;
            
            if (stricmp(pszAlignY, "center") == 0)
                nAlignY = ALIGN_CENTER;
            else if (stricmp(pszAlignY, "bottom") == 0)
                nAlignY = ALIGN_BOTTOM;
        }
    }
}

void DynamicBangCommand(HWND hCaller, const char *pszArguments)
{
    char pszLocation[MAX_BANGARGS];
    LPCSTR pszNextToken;
    
    static PopupMenu* pDynMenu = NULL;
    
    if (pDynMenu)
    {
        vector<PopupMenu*>::iterator iter =
            find(pPopupMaker->m_RootMenus.begin(), pPopupMaker->m_RootMenus.end(), pDynMenu);
        
        if (iter != pPopupMaker->m_RootMenus.end())
            pPopupMaker->m_RootMenus.erase(iter);

        delete pDynMenu;
    }

    if (GetToken(pszArguments, pszLocation, &pszNextToken, false))
    {
        int nX, nY, nAlignX, nAlignY;
        ParseBangParameters(pszNextToken, nX, nY, nAlignX, nAlignY);
        
        pDynMenu = pPopupMaker->LoadFolder(PathFindFileName(pszLocation), pszLocation);
        pPopupMaker->m_RootMenus.push_back(pDynMenu);
        pDynMenu->Show(nX, nY, nAlignX, nAlignY);
        SetForegroundWindow(pDynMenu->GetWindow());
    }
}

void ForceSetForegroundWindow(HWND hWnd)
{
   DWORD currentForeground = GetWindowThreadProcessId(GetForegroundWindow(), NULL);
   DWORD popupThread = GetCurrentThreadId();

   if (currentForeground != popupThread)
   {
      // If we're not the foreground, join it
      AttachThreadInput(currentForeground, popupThread, TRUE);
   }
   
   // Either way we need to bring our window to the front
   SetForegroundWindow(hWnd);
   SetFocus(hWnd); // Weird. Without this it only works the first time.

   if (currentForeground != popupThread)
   {
      // Detach the threads and put everything back like we found it.
      AttachThreadInput(currentForeground, popupThread, FALSE);
   }

   // Redraw the popup. Not sure if we need this, but doesn't hurt.
   InvalidateRect(hWnd, NULL, TRUE);
}

// handle !PopupXXX bang commands
void PopupBangCommand(HWND hCaller, const char *pszName, const char *pszArguments)
{
	int nX, nY, nAlignX, nAlignY;
    ParseBangParameters(pszArguments, nX, nY, nAlignX, nAlignY);

    for (int i = 0; i < pPopupMaker->m_RootMenus.size(); i++)
	{
		PopupMenu *pPopup = pPopupMaker->m_RootMenus[i];

		if (pPopup->GetBangName() && lstrcmpi(pPopup->GetBangName(), pszName) == 0)
		{
			pPopup->Show(nX, nY, nAlignX, nAlignY);
			ForceSetForegroundWindow(pPopup->GetWindow());
			return ;
		}
	}

	if (lstrcmpi(pszName, "!popup") == 0 && pPopupMaker->m_pRoot)
	{
		pPopupMaker->m_pRoot->Show(nX, nY, nAlignX, nAlignY);
		ForceSetForegroundWindow(pPopupMaker->m_pRoot->GetWindow());
		return ;
	}
}
